@@ -1,113 +1,143 @@
-"""The Unified Form Language is an embedded domain specific
-language for definition of variational forms intended for
-finite element discretization. More precisely, it defines
-a fixed interface for choosing finite element spaces and
-defining expressions for weak forms in a notation close
-to mathematical notation.
+"""The Unified Form Language is an embedded domain specific language
+for definition of variational forms intended for finite element
+discretization. More precisely, it defines a fixed interface for choosing
+finite element spaces and defining expressions for weak forms in a
+notation close to mathematical notation.
 
-This python module contains the language as well as
-algorithms to work with it.
+This python module contains the language as well as algorithms to work
+with it.
 
-- To import the language, type:
-      from ufl import *
+* To import the language, type::
 
-- To import the underlying classes an UFL expression tree
-  is built from, type:
-      from ufl.classes import *
+    from ufl import *
 
-- Various algorithms for working with UFL expression trees
-  can be found in:
-      from ufl.algorithms import *
+* To import the underlying classes an UFL expression tree is built
+  from, type::
 
-The classes and algorithms are considered implementation
-details and should not be used in form definitions.
+    from ufl.classes import *
 
-For more details on the language, see the manual or the
-wiki pages at
+* Various algorithms for working with UFL expression trees can be
+  found in::
 
-  http://www.fenics.org/wiki/UFL/
+    from ufl.algorithms import *
+
+The classes and algorithms are considered implementation details and
+should not be used in form definitions.
+
+For more details on the language, see
+
+  http://www.fenicsproject.org
 
 The development version can be found in the repository at
 
-  http://www.fenics.org/hg/ufl/
-
+  https://launchpad.net/ufl/
 
 A very brief overview of the language contents follows:
 
-Cells and Euclidean spaces:
+* Cells and Euclidean spaces::
+
     interval, triangle, tetrahedron, quadrilateral, hexahedron,
     R1, R2, R3,
     Cell, Space
 
-Elements:
+* Elements::
+
     FiniteElement, MixedElement, VectorElement, TensorElement
     EnrichedElement, RestrictedElement
 
-Arguments:
+* Arguments::
+
     Argument, TestFunction, TrialFunction
 
-Coefficients:
+* Coefficients::
+
     Coefficient, Constant, VectorConstant, TensorConstant
 
-Splitting form arguments in mixed spaces:
+* Splitting form arguments in mixed spaces::
+
     split
 
-Literal constants:
+* Literal constants::
+
     Identity
 
-Geometric quantities:
+* Geometric quantities::
+
     SpatialCoordinate, FacetNormal
 
-Indices:
+* Indices::
+
     Index, indices,
     i, j, k, l, p, q, r, s
 
-Scalar to tensor expression conversion:
+* Scalar to tensor expression conversion::
+
     as_tensor, as_vector, as_matrix
 
-Unit vectors and matrices:
+* Unit vectors and matrices::
+
     unit_vector, unit_vectors,
     unit_matrix, unit_matrices
 
-Tensor algebra operators:
+* Tensor algebra operators::
+
     outer, inner, dot, cross,
     transpose, tr,
     det, inv, cofac,
     dev, skew, sym,
 
-Differential operators:
+* Differential operators::
+
     variable, diff,
     Dx, grad, div, curl, rot, Dn
 
-Nonlinear functions:
+* Nonlinear functions::
+
     abs, sign, sqrt, exp, ln, cos, sin, tan, acos, asin, atan
 
-Discontinuous Galerkin operators:
+* Discontinuous Galerkin operators:
     jump, avg, v('+'), v('-')
 
-Conditional operators:
+* Conditional operators::
+
     eq, ne, le, ge, lt, gt, conditional,
 
-Integral measures:
+* Integral measures::
+
     dx, ds, dS, dE
 
-Form transformations:
+* Form transformations::
+
     rhs, lhs, system, functional,
     replace, adjoint, action, energy_norm,
     sensitivity_rhs, derivative
-
 """
 
-__version__ = "0.9.0"
-__authors__ = "Martin Sandve Alnes and Anders Logg"
-__copyright__ = "Copyright (C) 2008-2010 " + __authors__
-__license__  = "GNU GPL version 3 or any later version"
-__date__ = "2008-03-14"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Kristian Oelgaard, 2009
 # Modified by Anders Logg, 2009.
+#
+# First added:  2011-04-08
 # Last changed: 2010-02-01
 
+__version__ = "0.9.1"
+
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
 # which means we should _not_ import f.ex. "Grad", but "grad".
@@ -146,7 +176,7 @@ from ufl.coefficient import Coefficient, Coefficients, \
 from ufl.split_functions import split
 
 # Literal constants
-from ufl.constantvalue import Identity, Zero, as_ufl
+from ufl.constantvalue import PermutationSymbol, Identity, zero, Zero, as_ufl # TODO: Remove Zero
 
 # Indexing of tensor expressions
 from ufl.indexing import Index, indices
@@ -183,7 +213,8 @@ from ufl.integral import Integral, Measure, register_domain_type
 from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs, system, functional, adjoint, sensitivity_rhs #, dirichlet_functional
 
 # Predefined convenience objects
-from ufl.objects import vertex, interval, triangle, tetrahedron, quadrilateral, hexahedron, facet,\
+from ufl.objects import vertex, interval, triangle, tetrahedron, \
+    quadrilateral, hexahedron, facet, cell1D, cell2D, cell3D, \
     i, j, k, l, p, q, r, s, \
     R1, R2, R3, \
     dx, ds, dS, dE, dc
