@@ -1,12 +1,29 @@
 "This module collects algorithms and utility functions operating on UFL objects."
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-08-14 -- 2009-05-14"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008-2009.
+#
+# First added:  2008-08-14
+# Last changed: 2009-05-14
 
 # Function for preprocessing a form
-from ufl.algorithms.preprocess import preprocess
+from ufl.algorithms.preprocess import preprocess, extract_common_cell
 
 # Class for simple extraction of form meta data
 from ufl.algorithms.formdata import FormData
@@ -20,6 +37,7 @@ from ufl.algorithms.traversal import iter_expressions, traverse_terminals, \
 from ufl.algorithms.analysis import extract_classes, extract_type, has_type, \
                                     extract_arguments, extract_coefficients, extract_arguments_and_coefficients, \
                                     extract_elements, extract_unique_elements, \
+                                    extract_sub_elements, extract_unique_sub_elements, \
                                     extract_variables, extract_duplications, \
                                     extract_max_quadrature_element_degree, estimate_quadrature_degree, \
                                     sort_elements
