@@ -1,13 +1,29 @@
 """Utility algorithms for inspection of and information extraction from UFL objects in various ways."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-04-17"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009-2010.
 # Modified by Johan Hake, 2010.
-# Last changed: 2010-01-26
+#
+# First added:  2008-03-14
+# Last changed: 2011-05-12
 
-from itertools import chain
+from itertools import izip, chain
 
 from ufl.log import error, warning, info
 from ufl.assertions import ufl_assert
@@ -86,13 +102,13 @@ def extract_coefficients(a):
     which can be a Form, Integral or Expr."""
     return sorted(extract_type(a, Coefficient), cmp=cmp_counted)
 
-# FIXME: Why is this extra function needed, why not use
-# FIXME: the two functions above?
-
 def extract_arguments_and_coefficients(a):
     """Build two sorted lists of all arguments and coefficients
     in a, which can be a Form, Integral or Expr."""
 
+    # This function is faster than extract_arguments + extract_coefficients
+    # for large forms, and has more validation built in.
+
     # Extract lists of all form argument instances
     terminals = extract_type(a, FormArgument)
     arguments = [f for f in terminals if isinstance(f, Argument)]
@@ -104,14 +120,14 @@ def extract_arguments_and_coefficients(a):
 
     if len(bfcounts) != len(set(bfcounts.values())):
         msg = """\
-Found different basis function arguments with same counts.
+Found different Arguments with same counts.
 Did you combine test or trial functions from different spaces?
-The arguments found are:\n%s""" % "\n".join("  %s" % f for f in arguments)
+The Arguments found are:\n%s""" % "\n".join("  %s" % f for f in arguments)
         error(msg)
 
     if len(fcounts) != len(set(fcounts.values())):
         msg = """\
-Found different functions with same counts.
+Found different coefficients with same counts.
 The arguments found are:\n%s""" % "\n".join("  %s" % f for f in coefficients)
         error(msg)
 
@@ -121,17 +137,21 @@ The arguments found are:\n%s""" % "\n".join("  %s" % f for f in coefficients)
 
     return arguments, coefficients
 
-def build_argument_replace_map(arguments, coefficients):
+def build_argument_replace_map(arguments, coefficients, element_mapping=None):
     """Create new Argument and Coefficient objects
     with count starting at 0. Return mapping from old
     to new objects, and lists of the new objects."""
-    new_arguments = [f.reconstruct(count=i)\
-                           for (i, f) in enumerate(arguments)]
-    new_coefficients       = [f.reconstruct(count=i)\
-                           for (i, f) in enumerate(coefficients)]
-    replace_map = {}
-    replace_map.update(zip(arguments, new_arguments))
-    replace_map.update(zip(coefficients, new_coefficients))
+    if element_mapping is None:
+        element_mapping = {}
+    def remap(args):
+        for (i, f) in enumerate(args):
+            old_e = f.element()
+            new_e = element_mapping.get(old_e, old_e)
+            yield f.reconstruct(element=new_e, count=i)
+    new_arguments = list(remap(arguments))
+    new_coefficients = list(remap(coefficients))
+    replace_map = dict(izip(chain(arguments, coefficients),
+                            chain(new_arguments, new_coefficients)))
     return replace_map, new_arguments, new_coefficients
 
 # alternative implementation, kept as an example:
@@ -240,7 +260,7 @@ def extract_max_quadrature_element_degree(integral):
     return max_degree
 
 def estimate_quadrature_degree(integral):
-    "Estimate the necessary quadrature order for integral using the sum of basis function degrees."
+    "Estimate the necessary quadrature order for integral using the sum of argument degrees."
     arguments = extract_arguments(integral)
     degrees = [v.element().degree() for v in arguments]
     if len(arguments) == 0:
