@@ -1,19 +1,34 @@
 """This module defines utilities for transforming
 complete Forms into new related Forms."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-10-01"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008-2009.
 # Modified by Garth N. Wells, 2010.
 # Modified by Marie E. Rognes, 2010.
-
-# Last changed: 2010-11-17
+#
+# First added:  2008-10-01
+# Last changed: 2011-04-26
 
 from itertools import izip
 
 from ufl.common import some_key, product, Stack
-from ufl.log import error, warning
+from ufl.log import error, warning, debug
 from ufl.assertions import ufl_assert
 
 # All classes:
@@ -30,7 +45,6 @@ from ufl.classes import ufl_classes, terminal_classes, nonterminal_classes
 # Other algorithms:
 from ufl.algorithms.traversal import traverse_terminals
 from ufl.algorithms.analysis import extract_arguments
-from ufl.algorithms.preprocess import preprocess
 from ufl.algorithms.transformations import replace, Transformer, apply_transformer, transform_integrands
 
 def zero(e):
@@ -47,11 +61,11 @@ class PartExtracter(Transformer):
         self._want = set(arguments)
 
     def expr(self, x):
-        """The default is a nonlinear operator not accepting any basis
-        functions in its children."""
+        """The default is a nonlinear operator not accepting any
+        Arguments among its children."""
 
         if any(isinstance(t, Argument) for t in traverse_terminals(x)):
-            error("Found basis function in %s, this is an invalid expression." % repr(x))
+            error("Found Argument in %s, this is an invalid expression." % repr(x))
         return (x, set())
 
     # Terminals that are not Variables or Arguments behave as default
@@ -145,7 +159,7 @@ class PartExtracter(Transformer):
 
             # Throw error if size of sets are equal (and not zero)
             if len(provideds) == len(most_provided) and len(most_provided):
-                error("Don't know what to do with sums with different basis functions")
+                error("Don't know what to do with sums with different Arguments.")
 
             if provideds > most_provided:
                 most_provided = provideds
@@ -192,9 +206,9 @@ class PartExtracter(Transformer):
         # Get numerator and denominator
         numerator, denominator = x.operands()
 
-        # Check for basis functions in the denominator
+        # Check for Arguments in the denominator
         if any(isinstance(t, Argument) for t in traverse_terminals(denominator)):
-            error("Found basis function in denominator of %s , this is an invalid expression." % repr(x))
+            error("Found Argument in denominator of %s , this is an invalid expression." % repr(x))
 
         # Visit numerator
         numerator_parts, provides = self.visit(numerator)
@@ -211,8 +225,8 @@ class PartExtracter(Transformer):
         return (x, provides)
 
     def linear_operator(self, x, arg):
-        """A linear operator in a single argument accepting arity > 0,
-        providing whatever basis functions its argument does."""
+        """A linear operator with a single operand accepting arity > 0,
+        providing whatever Argument its operand does."""
 
         # linear_operator is a visit-children-first handler. Handled
         # arguments are in arg.
@@ -255,21 +269,29 @@ class PartExtracter(Transformer):
     spatial_derivative = linear_indexed_type
 
     def list_tensor(self, x, *ops):
-
         # list_tensor is a visit-children-first handler. ops contains
-        # the visited operands with their provides. Check that all
-        # provide the same arguments and return all
+        # the visited operands with their provides. (It follows that
+        # none of the visited operands provide more than wanted.)
 
-        default = ops[0][1]
-        for (items, provides) in ops:
-            if provides != default:
-                error("All components of a list tensor must provide same arguments")
+        # Extract the most arguments provided by any of the components
+        most_provides = ops[0][1]
+        for (component, provides) in ops:
+            if (provides - most_provides):
+                most_provides = provides
 
-        parts = [o[0] for o in ops]
+        # Check that all components either provide the same arguments
+        # or vanish. (This check is here b/c it is not obvious what to
+        # return if the components provide different arguments, at
+        # least with the current transformer design.)
+        for (component, provides) in ops:
+            if (provides != most_provides and not isinstance(component, Zero)):
+                error("PartExtracter does not know how to handle list_tensors with non-zero components providing fewer arguments")
 
-        x = self.reuse_if_possible(x, *parts)
+        # Return components
+        components = [op[0] for op in ops]
+        x = self.reuse_if_possible(x, *components)
 
-        return (x, default)
+        return (x, most_provides)
 
 def compute_form_with_arity(form, arity):
     """Compute parts of form of given arity."""
@@ -295,8 +317,8 @@ def compute_form_with_arity(form, arity):
 def compute_form_arities(form):
     """Return set of arities of terms present in form."""
 
-    # Preprocess form (preprocess takes care of checking)
-    form = preprocess(form)
+    # Need form to be preprocessed
+    ufl_assert(form.is_preprocessed(), "Expecting form to be preprocssed.")
 
     # Extract all arguments present in form
     arguments = extract_arguments(form)
@@ -334,46 +356,43 @@ def compute_form_rhs(form):
 
 def compute_form_functional(form):
     """Compute the functional part of a form, that
-    is the terms independent of basis functions.
+    is the terms independent of Arguments.
 
     (Used for testing, not sure if it's useful for anything?)"""
     return compute_form_with_arity(form, 0)
 
-def compute_form_action(form, function):
+def compute_form_action(form, coefficient):
     """Compute the action of a form on a Coefficient.
 
-    This works simply by replacing the last basis_function
+    This works simply by replacing the last Argument
     with a Coefficient on the same function space (element).
     The form returned will thus have one Argument less
-    and one additional Coefficient at the end if no function
-    has been provided.
+    and one additional Coefficient at the end if no
+    Coefficient has been provided.
     """
+    # Extract all arguments
     arguments = extract_arguments(form)
-    if len(arguments) == 2:
-        v, u = arguments
-    elif len(arguments) == 1:
-        u, = arguments
-    else:
-        error("Expecting bilinear or linear form.")
+
+    # Pick last argument (will be replaced)
+    u = arguments[-1]
 
     e = u.element()
-    if function is None:
-        function = Coefficient(e)
+    if coefficient is None:
+        coefficient = Coefficient(e)
     else:
-        #ufl_assert(function.element() == e, \
-        if function.element() != e:
-            print "Computing action of form on a " \
-                  "function in a different element space."
-    return replace(form, { u: function })
+        #ufl_assert(coefficient.element() == e, \
+        if coefficient.element() != e:
+            debug("Computing action of form on a coefficient in a different element space.")
+    return replace(form, { u: coefficient })
 
-def compute_energy_norm(form, function):
+def compute_energy_norm(form, coefficient):
     """Compute the a-norm of a Coefficient given a form a.
 
-    This works simply by replacing the two basis functions
+    This works simply by replacing the two Arguments
     with a Coefficient on the same function space (element).
     The Form returned will thus be a functional with no
-    basis functions, and one additional Coefficient at the
-    end if no function has been provided.
+    Arguments, and one additional Coefficient at the
+    end if no coefficient has been provided.
     """
     arguments = extract_arguments(form)
     ufl_assert(len(arguments) == 2, "Expecting bilinear form.")
@@ -381,13 +400,13 @@ def compute_energy_norm(form, function):
     e = u.element()
     e2 = v.element()
     ufl_assert(e == e2, "Expecting equal finite elements for test and trial functions, got '%s' and '%s'." % (str(e), str(e2)))
-    if function is None:
-        function = Coefficient(e)
+    if coefficient is None:
+        coefficient = Coefficient(e)
     else:
-        ufl_assert(function.element() == e, \
+        ufl_assert(coefficient.element() == e, \
             "Trying to compute action of form on a "\
-            "function in an incompatible element space.")
-    return replace(form, { u: function, v: function })
+            "coefficient in an incompatible element space.")
+    return replace(form, { u: coefficient, v: coefficient })
 
 def compute_form_adjoint(form):
     """Compute the adjoint of a bilinear form.
@@ -413,4 +432,3 @@ def compute_form_adjoint(form):
 #    #ufl_assert(len(bf) == 2, "Expecting bilinear form.")
 #    #v, u = bf
 #    #return replace(form, {u:v})
-
