@@ -1,10 +1,26 @@
 """Forward mode AD implementation."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-08-19"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009.
 # Modified by Garth N. Wells, 2010.
+#
+# First added:  2008-08-19
 # Last changed: 2010-06-07
 
 from ufl.log import error, warning, debug
@@ -371,6 +387,11 @@ class ForwardAD(Transformer):
     # --- Mathfunctions
 
     def math_function(self, o, a):
+        if hasattr(o, 'derivative'):
+            f, fp = a
+            o = self.reuse_if_possible(o, f)
+            op = fp * o.derivative()
+            return (o, op)
         error("Unknown math function.")
 
     def sqrt(self, o, a):
@@ -527,8 +548,8 @@ class SpatialAD(ForwardAD):
     constant = ForwardAD.terminal # returns zero
 
     def facet_normal(self, o):
-        if o.cell().degree() > 1:
-            warning("Treating facet normal as a constant in differentiation!")
+        #if o.cell().degree() > 1:
+        #    warning("Treating facet normal as a constant in differentiation!")
         return ForwardAD.terminal(self, o) # returns zero
 
 class VariableAD(ForwardAD):
@@ -558,8 +579,8 @@ class VariableAD(ForwardAD):
         self._variable_cache[l] = c
         return c
 
-class FunctionAD(ForwardAD):
-    "Apply AFD (Automatic Function Differentiation) to expression."
+class CoefficientAD(ForwardAD):
+    "Apply AFD (Automatic Functional Differentiation) to expression."
     def __init__(self, spatial_dim, coefficients, arguments, cache=None):
         ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=(), var_index_dimensions={}, cache=cache)
         self._functions = zip(coefficients, arguments)
@@ -570,7 +591,7 @@ class FunctionAD(ForwardAD):
         # Define dw/dw := v (what we really mean by d/dw is d/dw_j where w = sum_j w_j phi_j, and what we really mean by v is phi_j for any j)
 
     def coefficient(self, o):
-        debug("In FunctionAD.coefficient:")
+        debug("In CoefficientAD.coefficient:")
         debug("o = %s" % o)
         debug("self._w = %s" % self._w)
         debug("self._v = %s" % self._v)
@@ -610,9 +631,9 @@ def compute_variable_forward_ad(expr, dim):
     e, ediff = alg.visit(f)
     return ediff
 
-def compute_function_forward_ad(expr, dim):
+def compute_coefficient_forward_ad(expr, dim):
     f, w, v = expr.operands()
-    alg = FunctionAD(dim, w, v)
+    alg = CoefficientAD(dim, w, v)
     e, ediff = alg.visit(f)
     return ediff
 
@@ -625,7 +646,7 @@ def forward_ad(expr, dim):
     elif isinstance(expr, VariableDerivative):
         result = compute_variable_forward_ad(expr, dim)
     elif isinstance(expr, CoefficientDerivative):
-        result = compute_function_forward_ad(expr, dim)
+        result = compute_coefficient_forward_ad(expr, dim)
     else:
         error("This shouldn't happen: expr is %s" % repr(expr))
     return result
