@@ -1,7 +1,24 @@
 """Algorithms for working with linearized computational graphs."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-12-28 -- 2009-03-05"
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
+# First added:  2008-12-28
+# Last changed: 2009-03-05
 
 from collections import defaultdict
 from itertools import chain, imap, izip
@@ -283,31 +300,31 @@ class StringDependencyDefiner(MultiFunction):
     Possible dependency values are:
         "c"       - depends on runtime information like the cell, local<->global coordinate mappings, facet normals, or coefficients
         "x"       - depends on local coordinates
-        "v%d" % i - depends on basis function i, for i in [0,rank)
+        "v%d" % i - depends on argument i, for i in [0,rank)
     """
-    def __init__(self, basis_function_deps = None, function_deps = None):
+    def __init__(self, argument_deps = None, coefficient_deps = None):
         MultiFunction.__init__(self)
-        if basis_function_deps is None:
-            basis_function_deps = {}
-        if function_deps is None:
-            function_deps = {}
-        self.basis_function_deps = basis_function_deps
-        self.function_deps = function_deps
+        if argument_deps is None:
+            argument_deps = {}
+        if coefficient_deps is None:
+            coefficient_deps = {}
+        self.argument_deps = argument_deps
+        self.coefficient_deps = coefficient_deps
 
     def expr(self, o):
         return frozenset()
 
     def argument(self, x):
         default = frozenset(("v%d" % x.count(), "x"))
-        return self.basis_function_deps.get(x, default)
+        return self.argument_deps.get(x, default)
 
     def coefficient(self, x):
         default = frozenset(("c", "x"))
-        return self.function_deps.get(x, default)
+        return self.coefficient_deps.get(x, default)
 
     def constant(self, x):
         default = frozenset(("c",))
-        return self.function_deps.get(x, default)
+        return self.coefficient_deps.get(x, default)
 
     def geometric_quantity(self, x):
         deps = frozenset(("c", "x",))
@@ -317,15 +334,15 @@ class StringDependencyDefiner(MultiFunction):
         deps = frozenset(("c",))
         # Enabling coordinate dependency for higher order geometries
         # (not handled anywhere else though, so consider this experimental)
-        if o.cell().degree() > 1:
-            deps = deps | frozenset(("x",))
+        #if o.cell().degree() > 1:
+        #    deps = deps | frozenset(("x",))
         return deps
 
     def spatial_derivative(self, o): # TODO: What about (basis) functions of which derivatives are constant? Should special-case spatial_derivative in partition().
         deps = frozenset(("c",))
         # Enabling coordinate dependency for higher order geometries (not handled anywhere else though).
-        if o.cell().degree() > 1:
-            deps = deps | frozenset(("x",))
+        #if o.cell().degree() > 1:
+        #    deps = deps | frozenset(("x",))
         return deps
 
 dd = StringDependencyDefiner()
