@@ -1,7 +1,24 @@
 """Reverse mode AD implementation."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-12-28 -- 2009-01-07"
+# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
+# First added:  2008-12-28
+# Last changed: 2009-01-07
 
 # TODO: Imports!
 
