@@ -2,10 +2,26 @@
 either converting UFL expressions to new UFL expressions or
 converting UFL expressions to other representations."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-05-07 -- 2009-11-17"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009-2010
+#
+# First added:  2008-05-07
 # Last changed: 2010-10-25
 
 from itertools import izip
@@ -14,10 +30,9 @@ from inspect import getargspec
 from ufl.log import error, warning, debug, info
 from ufl.common import Stack, StackDict
 from ufl.assertions import ufl_assert
-from ufl.finiteelement import TensorElement
 from ufl.classes import Expr, Terminal, Product, Index, FixedIndex, ListTensor, Variable, Zero
 from ufl.indexing import indices, complete_shape
-from ufl.tensors import as_tensor, as_matrix, as_vector, ListTensor, ComponentTensor
+from ufl.tensors import as_tensor, as_matrix, as_vector
 from ufl.form import Form
 from ufl.integral import Integral
 from ufl.classes import all_ufl_classes
@@ -531,20 +546,20 @@ class NotMultiLinearException(Exception):
     def __init__(self, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
 
-class BasisFunctionDependencyExtracter(Transformer):
+class ArgumentDependencyExtracter(Transformer):
     def __init__(self):
         Transformer.__init__(self)
         self._empty = frozenset()
 
     def expr(self, o, *opdeps):
-        "Default for nonterminals: nonlinear in all arguments."
+        "Default for nonterminals: nonlinear in all operands."
         for d in opdeps:
             if d:
                 raise NotMultiLinearException, repr(o)
         return self._empty
 
     def terminal(self, o):
-        "Default for terminals: no dependency on basis functions."
+        "Default for terminals: no dependency on Arguments."
         return self._empty
 
     def variable(self, o):
@@ -602,7 +617,7 @@ class BasisFunctionDependencyExtracter(Transformer):
         return t
 
     def division(self, o, a, b):
-        "Basis functions cannot be in the denominator."
+        "Arguments cannot be in the denominator."
         if b:
             raise NotMultiLinearException, repr(o)
         return a
@@ -621,8 +636,8 @@ class BasisFunctionDependencyExtracter(Transformer):
             deps.update(d)
         return frozenset(deps)
 
-    # Product operands should not depend on the same basis functions
     def product(self, o, *opdeps):
+        # Product operands should not depend on the same Arguments
         c = []
         adeps, bdeps = opdeps # TODO: Generalize to any number of operands using permutations
         # for each frozenset ad in the frozenset adeps
@@ -816,6 +831,7 @@ def expand_compounds(e, dim=None):
     if dim is None:
         cell = e.cell()
         if cell is not None:
+            ufl_assert(not cell.is_undefined(), "Cannot infer dimension from undefined cell.")
             dim = cell.geometric_dimension()
     return apply_transformer(e, CompoundExpander(dim))
 
@@ -835,10 +851,10 @@ def purge_duplications(e):
     occur more than once with a single instance."""
     return apply_transformer(e, DuplicationPurger())
 
-def extract_basis_function_dependencies(e):
-    "Extract a set of sets of basis_functions."
+def extract_argument_dependencies(e):
+    "Extract a set of sets of Arguments."
     ufl_assert(isinstance(e, Expr), "Expecting an Expr.")
-    return BasisFunctionDependencyExtracter().visit(e)
+    return ArgumentDependencyExtracter().visit(e)
 
 def estimate_max_polynomial_degree(e, default_degree=1):
     """Estimate the maximum polymomial degree of all functions in the
