@@ -2,10 +2,27 @@
 either converting UFL expressions to new UFL expressions or
 converting UFL expressions to other representations."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-05-07 -- 2009-06-11"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008-2009.
+#
+# First added:  2008-05-07
+# Last changed: 2009-06-11
 
 import os
 from itertools import chain
@@ -77,7 +94,7 @@ def build_precedence_map():
 # Utility for parentesizing string
 def par(s, condition=True): # TODO: Finish precedence handling by adding condition argument to calls to this function!
     if condition:
-        return "\\left(%s\\right)" % s
+        return r"\left(%s\right)" % s
     return str(s)
 
 def format_index(ii):
@@ -117,19 +134,22 @@ class Expression2LatexHandler(Transformer):
         return "0" if not o.shape() else r"{\mathbf 0}"
 
     def identity(self, o):
-        return "{\mathbf I}"
+        return r"{\mathbf I}"
+
+    def permutation_symbol(self, o):
+        return r"{\mathbf \varepsilon}"
 
     def facet_normal(self, o):
-        return "{\mathbf n}"
+        return r"{\mathbf n}"
 
     def argument(self, o):
-        # Using ^ for function numbering and _ for indexing since indexing is more common than exponentiation
+        # Using ^ for argument numbering and _ for indexing since indexing is more common than exponentiation
         if self.argument_names is None:
             return bfname(o.count())
         return self.argument_names[o.count()]
 
     def coefficient(self, o):
-        # Using ^ for function numbering and _ for indexing since indexing is more common than exponentiation
+        # Using ^ for coefficient numbering and _ for indexing since indexing is more common than exponentiation
         if self.coefficient_names is None:
             return cfname(o.count())
         return self.coefficient_names[o.count()]
@@ -283,13 +303,13 @@ class Expression2LatexHandler(Transformer):
         return "(%s = %s)" % (a, b)
 
     def ne(self, o, a, b):
-        return "(%s \\ne %s)" % (a, b)
+        return r"(%s \ne %s)" % (a, b)
 
     def le(self, o, a, b):
-        return "(%s \\le %s)" % (a, b)
+        return r"(%s \le %s)" % (a, b)
 
     def ge(self, o, a, b):
-        return "(%s \\ge %s)" % (a, b)
+        return r"(%s \ge %s)" % (a, b)
 
     def lt(self, o, a, b):
         return "(%s < %s)" % (a, b)
@@ -341,22 +361,22 @@ def form2latex(form, formdata):
     # Define elements
     lines = []
     for i, f in enumerate(formdata.arguments):
-        lines.append("\\mathcal{P}_{%d} = \\{%s\\} " % (i, element2latex(f.element())))
+        lines.append(r"\mathcal{P}_{%d} = \{%s\} " % (i, element2latex(f.element())))
     for i, f in enumerate(formdata.coefficients):
-        lines.append("\\mathcal{Q}_{%d} = \\{%s\\} " % (i, element2latex(f.element())))
+        lines.append(r"\mathcal{Q}_{%d} = \{%s\} " % (i, element2latex(f.element())))
     if lines:
         sections.append(("Finite elements", align(lines)))
 
     # Define function spaces
     lines = []
     for i, f in enumerate(formdata.arguments):
-        lines.append("V_h^{%d} = \{v : v \\vert_K \in \\mathcal{P}_{%d}(K) \\quad \\forall K \in \\mathcal{T}\} " % (i, i))
+        lines.append("V_h^{%d} = \\{v : v \\vert_K \\in \\mathcal{P}_{%d}(K) \\quad \\forall K \\in \\mathcal{T}\\} " % (i, i))
     for i, f in enumerate(formdata.coefficients):
-        lines.append("W_h^{%d} = \{v : v \\vert_K \in \\mathcal{Q}_{%d}(K) \\quad \\forall K \in \\mathcal{T}\} " % (i, i))
+        lines.append("W_h^{%d} = \\{v : v \\vert_K \\in \\mathcal{Q}_{%d}(K) \\quad \\forall K \\in \\mathcal{T}\\} " % (i, i))
     if lines:
         sections.append(("Function spaces", align(lines)))
 
-    # Define basis functions and coefficients
+    # Define arguments and coefficients
     lines = []
     for i, f in enumerate(formdata.arguments):
         lines.append("%s = %s \\in V_h^{%d} " % (argument_names[i], bfname(i), i))
@@ -403,18 +423,19 @@ def form2latex(form, formdata):
         b = p + "\\int_{%s_%d}" % (dstr, itg.measure().domain_id())
         dxstr = Measure._domain_types[dtyp]
         dxstr = dx_strings[dtyp]
-        c = "{ %s } \,%s" % (integrand_string, dxstr)
+        c = "{ %s } \\,%s" % (integrand_string, dxstr)
         lines.append((a, b, c))
         a = "{}"; p = "{}+ "
     sections.append(("Form", align(lines)))
 
     return sections
 
-def ufl2latex(expression, form_data=None):
+def ufl2latex(expression):
     "Generate LaTeX code for a UFL expression or form (wrapper for form2latex and expression2latex)."
     if isinstance(expression, Form):
-        form = preprocess(expression)
-        return form2latex(form, form.form_data())
+        form_data = expression.compute_form_data()
+        preprocessed_form = form_data.preprocessed_form
+        return form2latex(preprocessed_form, form_data)
     else:
         return expression2latex(expression)
 
@@ -449,7 +470,7 @@ def dependency_sorting(deplist, rank):
     precompute_quad, left = split(left, state)
     deplistlist.append(precompute_quad)
 
-    # Permutations of 0/1 dependence of basis functions
+    # Permutations of 0/1 dependence of arguments
     indices = compute_indices((2,)*rank)
     for bfs in indices[1:]: # skip (0,...,0), already handled that
         for i, bf in reversed(list(enumerate(bfs))):
