@@ -1,11 +1,27 @@
 """This module defines the class Argument and a number of related
 classes (functions), including TestFunction and TrialFunction."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-02-23"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008-2009.
-# Last changed: 2009-12-08
+#
+# First added:  2008-03-14
+# Last changed: 2011-05-12
 
 from ufl.assertions import ufl_assert
 from ufl.common import Counted, product
@@ -27,10 +43,20 @@ class Argument(FormArgument, Counted):
         self._element = element
         self._repr = "Argument(%r, %r)" % (self._element, self._count)
 
-    def reconstruct(self, count=None):
+    def reconstruct(self, element=None, count=None):
+        if element is None or element == self._element:
+            element = self._element
         if count is None or count == self._count:
+            count = self._count
+        if count is self._count and element is self._element:
             return self
-        return Argument(self.element(), count)
+        ufl_assert(isinstance(element, FiniteElementBase),
+                   "Expecting an element, not %s" % element)
+        ufl_assert(isinstance(count, int),
+                   "Expecting an int, not %s" % count)
+        ufl_assert(element.value_shape() == self._element.value_shape(),
+                   "Cannot reconstruct a VectorConstant with a different value shape.")
+        return Argument(element, count)
 
     def element(self):
         return self._element
