@@ -2,16 +2,32 @@
 since it enables the syntax "from ufl.classes import FooBar" for getting
 implementation details not exposed through the default ufl namespace."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-08-15"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009.
+#
+# First added:  2008-08-15
 # Last changed: 2009-12-08
 
 from ufl.assertions import ufl_assert
 from ufl.expr import Expr, Operator, WrapperType, AlgebraOperator
 from ufl.terminal import Terminal, FormArgument, UtilityType, Tuple
-from ufl.constantvalue import ConstantValue, Zero, ScalarValue, FloatValue, IntValue, Identity
+from ufl.constantvalue import ConstantValue, Zero, ScalarValue, FloatValue, IntValue, Identity, PermutationSymbol
 from ufl.variable import Variable, Label
 from ufl.finiteelement import FiniteElementBase, FiniteElement, MixedElement, VectorElement, TensorElement
 from ufl.argument import Argument, TestFunction, TrialFunction
