@@ -1,10 +1,27 @@
 """This module defines the Coefficient class and a number
-of related classes (functions), including Constant."""
-
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-06-17"
+of related classes, including Constant."""
 
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008-2009.
+#
+# First added:  2008-03-14
+# Last changed: 2011-05-11
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
@@ -35,10 +52,25 @@ class Coefficient(FormArgument, Counted):
         #    # TODO: Check shapes of gradient and derivatives
         #    warning("Specifying the gradient or derivatives of a Coefficient is not yet implemented anywhere.")
 
-    def reconstruct(self, count=None):
+    def reconstruct(self, element=None, count=None):
+        # This code is shared with the FooConstant classes
+        if element is None or element == self._element:
+            element = self._element
         if count is None or count == self._count:
+            count = self._count
+        if count is self._count and element is self._element:
             return self
-        return Coefficient(self._element, count)
+        ufl_assert(isinstance(element, FiniteElementBase),
+                   "Expecting an element, not %s" % element)
+        ufl_assert(isinstance(count, int),
+                   "Expecting an int, not %s" % count)
+        ufl_assert(element.value_shape() == self._element.value_shape(),
+                   "Cannot reconstruct a VectorConstant with a different value shape.")
+        return self._reconstruct(element, count)
+
+    def _reconstruct(self, element, count):
+        # This code is class specific
+        return Coefficient(element, count)
 
     #def gradient(self):
     #    "Hook for experimental feature, do not use!"
@@ -72,7 +104,7 @@ class Coefficient(FormArgument, Counted):
     def __eq__(self, other):
         return isinstance(other, Coefficient) and self._element == other._element and self._count == other._count
 
-# --- Subclasses for defining constant functions without specifying element ---
+# --- Subclasses for defining constant coefficients without specifying element ---
 
 # TODO: Handle actual global constants?
 
@@ -89,10 +121,8 @@ class Constant(ConstantBase):
         ConstantBase.__init__(self, e, count)
         self._repr = "Constant(%r, %r)" % (e.cell(), self._count)
 
-    def reconstruct(self, count=None):
-        if count is None or count == self._count:
-            return self
-        return Constant(self._element.cell(), count)
+    def _reconstruct(self, element, count):
+        return Constant(element.cell(), count)
 
     def __str__(self):
         count = str(self._count)
@@ -109,11 +139,8 @@ class VectorConstant(ConstantBase):
         ConstantBase.__init__(self, e, count)
         self._repr = "VectorConstant(%r, %r, %r)" % (e.cell(), e.value_shape()[0], self._count)
 
-    def reconstruct(self, count=None):
-        if count is None or count == self._count:
-            return self
-        e = self._element
-        return VectorConstant(e.cell(), e.value_shape()[0], count)
+    def _reconstruct(self, element, count):
+        return VectorConstant(element.cell(), element.value_shape()[0], count)
 
     def __str__(self):
         count = str(self._count)
@@ -129,10 +156,8 @@ class TensorConstant(ConstantBase):
         ConstantBase.__init__(self, e, count)
         self._repr = "TensorConstant(%r, %r, %r, %r)" % (e.cell(), e.value_shape(), e._symmetry, self._count)
 
-    def reconstruct(self, count=None):
-        if count is None or count == self._count:
-            return self
-        e = self._element
+    def _reconstruct(self, element, count):
+        e = element
         return TensorConstant(e.cell(), e.value_shape(), e._symmetry, count)
 
     def __str__(self):
