@@ -1,9 +1,25 @@
 "Differential operators."
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009.
+#
+# First added:  2008-03-14
 # Last changed: 2009-12-08
 
 from ufl.log import warning, error
@@ -27,7 +43,7 @@ def is_spatially_constant(expression):
     "Check if a terminal object is spatially constant, such that expression.dx(i) == 0."
     if isinstance(expression, (ConstantValue, ConstantBase)):
         return True
-    elif isinstance(expression, FacetNormal) and expression.cell().degree() == 1:
+    elif isinstance(expression, FacetNormal): # and expression.cell().degree() == 1:
         return True
     elif isinstance(expression, Coefficient):
         e = expression.element()
@@ -49,27 +65,27 @@ class Derivative(Operator):
 class CoefficientDerivative(Derivative):
     """Derivative of the integrand of a form w.r.t. the
     degrees of freedom in a discrete Coefficient."""
-    __slots__ = ("_integrand", "_functions", "_basis_functions")
+    __slots__ = ("_integrand", "_coefficients", "_arguments")
 
-    def __new__(cls, integrand, functions, basis_functions):
+    def __new__(cls, integrand, coefficients, arguments):
         ufl_assert(is_true_ufl_scalar(integrand),
             "Expecting true UFL scalar expression.")
-        ufl_assert(isinstance(functions, Tuple), #and all(isinstance(f, (Coefficient,Indexed)) for f in functions),
+        ufl_assert(isinstance(coefficients, Tuple), #and all(isinstance(f, (Coefficient, Indexed)) for f in coefficients),
             "Expecting Tuple instance with Coefficients.")
-        ufl_assert(isinstance(basis_functions, Tuple), #and all(isinstance(f, Argument) for f in basis_functions),
+        ufl_assert(isinstance(arguments, Tuple), #and all(isinstance(f, Argument) for f in arguments),
             "Expecting Tuple instance with Arguments.")
         if isinstance(integrand, Zero):
             return Zero()
         return Derivative.__new__(cls)
 
-    def __init__(self, integrand, functions, basis_functions):
+    def __init__(self, integrand, coefficients, arguments):
         Derivative.__init__(self)
         self._integrand = integrand
-        self._functions = functions
-        self._basis_functions = basis_functions
+        self._coefficients = coefficients
+        self._arguments = arguments
 
     def operands(self):
-        return (self._integrand, self._functions, self._basis_functions)
+        return (self._integrand, self._coefficients, self._arguments)
 
     def shape(self):
         return ()
@@ -81,16 +97,19 @@ class CoefficientDerivative(Derivative):
         return {}
 
     def __str__(self):
-        return "CoefficientDerivative (w.r.t. function %s and using basis function %s) of \n%s" % (self._functions, self._basis_functions, self._integrand) # TODO: Short notation
+        # TODO: Short str notation
+        return "CoefficientDerivative (w.r.t. coefficient %s and using argument %s) of \n%s"\
+            % (self._coefficients, self._arguments, self._integrand)
 
     def __repr__(self):
-        return "CoefficientDerivative(%r, %r, %r)" % (self._integrand, self._functions, self._basis_functions)
+        return "CoefficientDerivative(%r, %r, %r)"\
+            % (self._integrand, self._coefficients, self._arguments)
 
 def split_indices(expression, idx):
     idims = dict(expression.index_dimensions())
     if isinstance(idx, Index) and idims.get(idx) is None:
         cell = expression.cell()
-        ufl_assert(cell is not None,
+        ufl_assert(cell is not None and not cell.is_undefined(),
             "Need to know the spatial dimension to "\
             "compute the shape of derivatives.")
         idims[idx] = cell.geometric_dimension()
@@ -115,9 +134,8 @@ class SpatialDerivative(Derivative):
 
         # Make a MultiIndex with knowledge of the dimensions
         cell = expression.cell()
-        if cell is None:
-            sh = None
-            error("Cannot compute derivatives of expressions with no cell.")
+        if cell is None or cell.is_undefined():
+            error("Cannot compute derivatives of expressions with undefined cell.")
         else:
             sh = (cell.geometric_dimension(),)
         self._index = as_multi_index(index, sh)
@@ -179,7 +197,7 @@ class VariableDerivative(Derivative):
     def __new__(cls, f, v):
         # Return zero if expression is trivially independent of Coefficient
         if isinstance(f, Terminal):# and not isinstance(f, Variable):
-            free_indices = set(f.free_indices()) ^ set(v.free_indices())
+            free_indices = tuple(set(f.free_indices()) ^ set(v.free_indices()))
             index_dimensions = mergedicts([f.index_dimensions(), v.index_dimensions()])
             index_dimensions = subdict(index_dimensions, free_indices)
             return Zero(f.shape() + v.shape(), free_indices, index_dimensions)
@@ -245,23 +263,27 @@ class Grad(CompoundDerivative):
 
     def __new__(cls, f):
         # Return zero if expression is trivially constant
+        cell = f.cell()
+        ufl_assert(cell is not None and not cell.is_undefined(),\
+                   "Can't take gradient of expression with undefined cell...")
+        dim = cell.geometric_dimension()
         if is_spatially_constant(f):
-            cell = f.cell()
-            ufl_assert(cell is not None, "Can't take gradient of expression with undefined cell...")
-            dim = cell.geometric_dimension()
             free_indices = f.free_indices()
             index_dimensions = subdict(f.index_dimensions(), free_indices)
             return Zero(f.shape() + (dim,), free_indices, index_dimensions)
+        if dim == 1:
+            return f.dx(0)
         return CompoundDerivative.__new__(cls)
 
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         self._f = f
         cell = f.cell()
-        ufl_assert(cell is not None, "Can't take gradient of expression with undefined cell. How did this happen?")
+        ufl_assert(cell is not None and not cell.is_undefined(),\
+                   "Can't take gradient of expression with undefined cell...")
         self._dim = cell.geometric_dimension()
-        ufl_assert(not (f.free_indices()), \
-            "TODO: Taking gradient of an expression with free indices, should this be a valid expression? Please provide examples!")
+        ufl_assert(not f.free_indices(),\
+            "Taking gradient of an expression with free indices is not supported.")
         self._repr = "Grad(%r)" % self._f
 
     def operands(self):
@@ -286,12 +308,18 @@ class Div(CompoundDerivative):
     __slots__ = ("_f", "_repr")
 
     def __new__(cls, f):
-        ufl_assert(f.rank() >= 1, "Can't take the divergence of a scalar.")
-        ufl_assert(not (f.free_indices()), \
-            "TODO: Taking divergence of an expression with free indices, should this be a valid expression? Please provide examples!")
+        ufl_assert(not f.free_indices(), \
+            "TODO: Taking divergence of an expression with free indices,"\
+            "should this be a valid expression? Please provide examples!")
+
+        if f.rank() == 0:
+            return f.dx(0)
+        #ufl_assert(f.rank() >= 1, "Can't take the divergence of a scalar.")
+
         # Return zero if expression is trivially constant
         if is_spatially_constant(f):
-            return Zero(f.shape()[:-1]) # No free indices
+            return Zero(f.shape()[:-1]) # No free indices asserted above
+
         return CompoundDerivative.__new__(cls)
 
     def __init__(self, f):
@@ -330,7 +358,8 @@ class Curl(CompoundDerivative):
         # Return zero if expression is trivially constant
         if is_spatially_constant(f):
             cell = f.cell()
-            ufl_assert(cell is not None, "Can't take curl of expression with undefined cell...")
+            ufl_assert(cell is not None and not cell.is_undefined(),\
+                       "Can't take curl of expression with undefined cell...")
             sh = { (): (2,), (2,): (), (3,): (3,) }[sh]
             #free_indices = f.free_indices()
             #index_dimensions = subdict(f.index_dimensions(), free_indices)
@@ -341,7 +370,8 @@ class Curl(CompoundDerivative):
     def __init__(self, f):
         CompoundDerivative.__init__(self)
         cell = f.cell()
-        ufl_assert(cell is not None, "Can't take curl of expression with undefined cell...")
+        ufl_assert(cell is not None and not cell.is_undefined(),\
+                   "Can't take curl of expression with undefined cell...")
         sh = { (): (2,), (2,): (), (3,): (3,) }[f.shape()]
         self._f = f
         self._shape = sh
