@@ -1,3 +1,20 @@
+# Copyright (C) 2010-2011 Marie E. Rognes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+
 from ufl.assertions import ufl_assert
 
 def FEEC_aliases(name, cell, r, k):
@@ -23,12 +40,14 @@ def FEEC_aliases(name, cell, r, k):
     Numerica, 2006, Table 5.1 and 5.2 (p. 60)
     """
 
-    ufl_assert(k in set(range(0, cell.topological_dimension()+1)),
+    ufl_assert(not cell.is_undefined(), "Cannot get dimension from undefined cell.")
+    tdim = cell.topological_dimension()
+    ufl_assert(k in set(range(0, tdim+1)),\
                "k-forms only defined for k in [0, n]")
 
     if k == 0:
         family = "CG"
-    elif k == cell.topological_dimension():
+    elif k == tdim:
         family = "DG"
         if name == "P- Lambda":
             r = r - 1
@@ -37,7 +56,7 @@ def FEEC_aliases(name, cell, r, k):
             family = "Nedelec 2nd kind H(curl)"
         elif name == "P- Lambda":
             family = "Nedelec 1st kind H(curl)"
-    elif k == cell.topological_dimension() - 1:
+    elif k == tdim - 1:
         if name == "P Lambda":
             family = "Nedelec 2nd kind H(div)"
         elif name == "P- Lambda":
