@@ -1,26 +1,43 @@
 "This module defines the UFL finite element classes."
 
-__authors__ = "Martin Sandve Alnes and Anders Logg"
-__date__ = "2008-03-03"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Kristian B. Oelgaard
-# Modified by Marie E. Rognes (meg@simula.no) 2010
-
-# Last changed: 2010-12-08
+# Modified by Marie E. Rognes <meg@simula.no> 2010
+#
+# First added:  2008-03-03
+# Last changed: 2011-05-08
 
+from itertools import izip
 from ufl.assertions import ufl_assert
 from ufl.permutation import compute_indices
 from ufl.elementlist import ufl_elements, aliases
 from ufl.common import product, index_to_component, component_to_index, istr
 from ufl.geometry import as_cell, domain2facet
-from ufl.log import info_blue
+from ufl.log import info_blue, warning
 from ufl.log import BLUE
 
 class FiniteElementBase(object):
     "Base class for all finite elements"
-    __slots__ = ("_family", "_cell", "_degree", "_value_shape", "_repr", "_domain")
+    __slots__ = ("_family", "_cell", "_degree", "_quad_scheme", "_value_shape",
+                 "_repr", "_domain")
 
-    def __init__(self, family, cell, degree, value_shape):
+    def __init__(self, family, cell, degree, quad_scheme, value_shape):
         "Initialize basic finite element data"
         ufl_assert(isinstance(family, str), "Invalid family type.")
         cell = as_cell(cell)
@@ -31,6 +48,7 @@ class FiniteElementBase(object):
         self._degree = degree
         self._value_shape = value_shape
         self._domain = None
+        self._quad_scheme = quad_scheme
 
     def family(self):
         "Return finite element family"
@@ -40,17 +58,13 @@ class FiniteElementBase(object):
         "Return cell of finite element"
         return self._cell
 
-    def set_cell(self, cell):
-        "Set cell for element"
-        self._cell = cell
-
     def degree(self):
         "Return polynomial degree of finite element"
         return self._degree
 
-    def set_degree(self, degree):
-        "Set degree for element"
-        self._degree = degree
+    def quadrature_scheme(self):
+        "Return quadrature scheme of finite element"
+        return self._quad_scheme
 
     def value_shape(self):
         "Return the shape of the value space"
@@ -108,7 +122,9 @@ class FiniteElementBase(object):
         "Restrict finite element to a subdomain, subcomponent or topology (cell)."
         from ufl.integral import Measure
         from ufl.geometry import Cell
-        if isinstance(index, Measure) or isinstance(as_cell(index), Cell):
+        if isinstance(index, (Measure, Cell)) or\
+                index == "facet" or\
+                isinstance(as_cell(index), Cell): # TODO: Can we just drop the as_cell call?
             return RestrictedElement(self, index)
         #if isinstance(index, int):
         #    return SubElement(self, index)
@@ -116,56 +132,90 @@ class FiniteElementBase(object):
 
 class FiniteElement(FiniteElementBase):
     "The basic finite element class for all simple finite elements"
-
-    def __init__(self, family, cell, degree=None, form_degree=None):
+    def __init__(self, family, cell=None, degree=None, quad_scheme=None,
+                 form_degree=None):
         "Create finite element"
 
+        # Map evt. string argument to a Cell
         cell = as_cell(cell)
-        domain = cell.domain()
 
         # Check whether this family is an alias for something else
         if family in aliases:
             (name, cell, r) = aliases[family](family, cell, degree, form_degree)
-            info_blue("%s, is an alias for %s " % ((family, cell, degree, form_degree),
-                                                   (name, cell, r)))
-            self.__init__(name, cell, r)
+            info_blue("%s, is an alias for %s " % (
+                    (family, cell, degree, form_degree),
+                    (name, cell, r)))
+            # FIXME: Missing form_degree here? What is form_degree?
+            self.__init__(name, cell, r, quad_scheme)
             return
 
         # Check that the element family exists
-        ufl_assert(family in ufl_elements, 'Unknown finite element "%s".' % family)
+        ufl_assert(family in ufl_elements,
+                   'Unknown finite element "%s".' % family)
 
         # Check that element data is valid (and also get common family name)
-        (family, self._short_name, value_rank, krange, domains) = ufl_elements[family]
+        (family, self._short_name, value_rank, krange, domains) =\
+            ufl_elements[family]
 
-        ufl_assert(domain in domains or domain is None,
-                   'Domain "%s" invalid for "%s" finite element.' % (domain, family))
-        if krange is None:
-            ufl_assert(degree is None,
-                       'Degree "%s" invalid for "%s" finite element, should be None.' % (istr(degree), family))
+        # Validate domain if a valid cell is specified
+        if cell.is_undefined():
+            # Case of invalid cell, some stuff is then undefined,
+            # such as the domain and some dimensions
+            pass
         else:
+            domain = cell.domain()
+            ufl_assert(domain in domains,
+                       'Domain "%s" invalid for "%s" finite element.' % (domain, family))
+
+        # Validate degree if specified
+        if degree is not None:
+            ufl_assert(krange is not None,
+                       'Degree "%s" invalid for "%s" finite element, '\
+                           'should be None.' % (degree, family))
             kmin, kmax = krange
-            ufl_assert(kmin is None or degree >= kmin or degree is None,
-                       'Degree "%s" invalid for "%s" finite element.' % (istr(degree), family))
-            ufl_assert(kmax is None or degree <= kmax or degree is None,
-                   'Degree "%s" invalid for "%s" finite element.' % (istr(degree), family))
+            ufl_assert(kmin is None or degree >= kmin,
+                       'Degree "%s" invalid for "%s" finite element.' %\
+                           (degree, family))
+            ufl_assert(kmax is None or degree <= kmax,
+                   'Degree "%s" invalid for "%s" finite element.' %\
+                           (istr(degree), family))
 
         # Set value dimension (default to using domain dimension in each axis)
-        dim = cell.geometric_dimension()
-        value_shape = (dim,)*(value_rank)
+        if value_rank == 0:
+            value_shape = ()
+        else:
+            ufl_assert(not cell.is_undefined(),
+                       "Cannot infer value shape with an undefined cell.")
+            dim = cell.geometric_dimension()
+            value_shape = (dim,)*value_rank
 
         # Initialize element data
-        FiniteElementBase.__init__(self, family, cell, degree, value_shape)
+        super(FiniteElement, self).__init__(family, cell, degree,
+                                            quad_scheme, value_shape)
 
         # Cache repr string
-        self._repr = "FiniteElement(%r, %r, %s)" % (self.family(), self.cell(), istr(self.degree()))
+        self._repr = "FiniteElement(%r, %r, %r, %r)" % (
+            self.family(), self.cell(), self.degree(), self.quadrature_scheme())
+
+    def reconstruct(self, **kwargs):
+        """Construct a new FiniteElement object with some properties
+        replaced with new values."""
+        kwargs["family"] = kwargs.get("family", self.family())
+        kwargs["cell"] = kwargs.get("cell", self.cell())
+        kwargs["degree"] = kwargs.get("degree", self.degree())
+        kwargs["quad_scheme"] = kwargs.get("quad_scheme", self.quadrature_scheme())
+        return FiniteElement(**kwargs)
 
     def __str__(self):
         "Format as string for pretty printing."
-        return "<%s%s on a %s>" % (self._short_name, istr(self.degree()), self.cell())
+        qs = self.quadrature_scheme()
+        qs = "" if qs is None else "(%s)" % qs
+        return "<%s%s%s on a %s>" % (self._short_name, istr(self.degree()),\
+                                           qs, self.cell())
 
     def shortstr(self):
         "Format as string for pretty printing."
-        return "%s%s" % (self._short_name, istr(self.degree()))
+        return "%s%s(%s)" % (self._short_name, istr(self.degree()), istr(self.quadrature_scheme()))
 
 class MixedElement(FiniteElementBase):
     "A finite element composed of a nested hierarchy of mixed or simple elements"
@@ -181,50 +231,69 @@ class MixedElement(FiniteElementBase):
 
         # Check that all elements are defined on the same domain
         cell = elements[0].cell()
-        ufl_assert(all(e.cell() == cell for e in elements), "Cell mismatch for sub elements of mixed element.")
+        ufl_assert(all(e.cell() == cell for e in elements),
+                   "Cell mismatch for sub elements of mixed element.")
+
+        # Check that all elements use the same quadrature scheme
+        # TODO: We can allow the scheme not to be defined.
+        quad_scheme = elements[0].quadrature_scheme()
+        ufl_assert(all(e.quadrature_scheme() == quad_scheme for e in elements),\
+            "Quadrature scheme mismatch for sub elements of mixed element.")
 
         # Compute value shape
-        value_size_sum= sum(product(s.value_shape()) for s in self._sub_elements)
-        if "value_shape" in kwargs:
-            value_shape = kwargs["value_shape"]
-            # Validate value_shape
-            if type(self) is MixedElement:
-                ufl_assert(product(value_shape) == value_size_sum,
-                    "Provided value_shape doesn't match the total "\
-                    "value size of all subelements.")
-        else:
-            # Default value dimension: Treated simply as all subelement values unpacked in a vector.
-            value_shape = (value_size_sum,)
+        value_size_sum = sum(product(s.value_shape()) for s in self._sub_elements)
+        # Default value dimension: Treated simply as all subelement values unpacked in a vector.
+        value_shape = kwargs.get('value_shape', (value_size_sum,))
+        # Validate value_shape
+        if type(self) is MixedElement:
+            # This is not valid for tensor elements with symmetries,
+            # assume subclasses deal with their own validation
+            ufl_assert(product(value_shape) == value_size_sum,
+                "Provided value_shape doesn't match the total "\
+                "value size of all subelements.")
 
         # Initialize element data
         degree = max(e.degree() for e in self._sub_elements)
-        FiniteElementBase.__init__(self, "Mixed", cell, degree, value_shape)
+        super(MixedElement, self).__init__("Mixed", cell, degree,
+                                           quad_scheme, value_shape)
 
         # Cache repr string
-        self._repr = "MixedElement(*%r, **{'value_shape': %r })" % (self._sub_elements, self._value_shape)
+        self._repr = "MixedElement(*%r, **{'value_shape': %r })" %\
+            (self._sub_elements, self._value_shape)
 
-    def set_cell(self, cell):
-        "Set cell for element"
-        self._cell = cell
-        for element in self._sub_elements:
-            element.set_cell(cell)
+    def reconstruct(self, **kwargs):
+        """Construct a new MixedElement object with some properties
+        replaced with new values."""
+        elements = [e.reconstruct(**kwargs) for e in self._sub_elements]
+        # Value shape cannot be changed, or at
+        # least we have no valid use case for it.
+        # Reconstructing an expression with a reconstructed
+        # coefficient with a different value shape would
+        # be way into undefined behaviour territory...
+        ufl_assert("value_shape" not in kwargs,
+                   "Cannot change value_shape in reconstruct.")
+        return self.reconstruct_from_elements(*elements)
 
-    def set_degree(self, degree):
-        "Set degree for element"
-        self._degree = degree
-        for element in self._sub_elements:
-            element.set_degree(degree)
+    def reconstruct_from_elements(self, *elements):
+        "Reconstruct a mixed element from new subelements."
+        if all(a == b for (a,b) in izip(elements, self._sub_elements)):
+            return self
+        ufl_assert(all(a.value_shape() == b.value_shape()
+                       for (a,b) in izip(elements, self._sub_elements)),
+            "Expecting new elements to have same value shape as old ones.")
+        return MixedElement(*elements, value_shape=self.value_shape())
 
     def num_sub_elements(self):
-        "Return number of sub elements"
+        "Return number of sub elements."
         return len(self._sub_elements)
 
     def sub_elements(self):
-        "Return list of sub elements"
+        "Return list of sub elements."
         return self._sub_elements
 
     def extract_component(self, i):
-        "Extract base component index and (simple) element for given component index"
+        """Extract base component index and (simple) element
+        for given component index."""
         if isinstance(i, int):
             i = (i,)
         self._check_component(i)
@@ -261,17 +330,18 @@ class MixedElement(FiniteElementBase):
 class VectorElement(MixedElement):
     "A special case of a mixed finite element where all elements are equal"
 
-    def __init__(self, family, cell, degree, dim=None):
+    def __init__(self, family, cell, degree, dim=None, quad_scheme=None):
         "Create vector element (repeated mixed element)"
 
         cell = as_cell(cell)
 
         # Set default size if not specified
         if dim is None:
+            ufl_assert(not cell.is_undefined(), "Cannot infer dimension with an undefined cell.")
             dim = cell.geometric_dimension()
 
         # Create mixed element from list of finite elements
-        sub_element = FiniteElement(family, cell, degree)
+        sub_element = FiniteElement(family, cell, degree, quad_scheme)
         sub_elements = [sub_element]*dim
 
         # Get common family name (checked in FiniteElement.__init__)
@@ -281,13 +351,23 @@ class VectorElement(MixedElement):
         value_shape = (dim,) + sub_element.value_shape()
 
         # Initialize element data
-        MixedElement.__init__(self, sub_elements, value_shape=value_shape)
+        super(VectorElement, self).__init__(sub_elements, value_shape=value_shape)
         self._family = family
         self._degree = degree
         self._sub_element = sub_element
 
-        self._repr = "VectorElement(%r, %r, %s, %d)" % \
-            (self._family, self._cell, str(self._degree), len(self._sub_elements))
+        # Cache repr string
+        self._repr = "VectorElement(%r, %r, %r, %d, %r)" % \
+            (self._family, self._cell, self._degree, len(self._sub_elements), quad_scheme)
+
+    def reconstruct(self, **kwargs):
+        kwargs["family"] = kwargs.get("family", self.family())
+        kwargs["cell"] = kwargs.get("cell", self.cell())
+        kwargs["degree"] = kwargs.get("degree", self.degree())
+        ufl_assert("dim" not in kwargs, "Cannot change dim in reconstruct.")
+        kwargs["dim"] = len(self._sub_elements)
+        kwargs["quad_scheme"] = kwargs.get("quad_scheme", self.quadrature_scheme())
+        return VectorElement(**kwargs)
 
     def __str__(self):
         "Format as string for pretty printing."
@@ -303,12 +383,13 @@ class TensorElement(MixedElement):
     #__slots__ = ("_family", "_cell", "_degree", "_value_shape")
     __slots__ = ("_sub_element", "_shape", "_symmetry", "_sub_element_mapping",)
 
-    def __init__(self, family, cell, degree, shape=None, symmetry=None):
+    def __init__(self, family, cell, degree, shape=None, symmetry=None, quad_scheme=None):
         "Create tensor element (repeated mixed element with optional symmetries)"
         cell = as_cell(cell)
 
         # Set default shape if not specified
         if shape is None:
+            ufl_assert(not cell.is_undefined(), "Cannot infer value shape with an undefined cell.")
             dim = cell.geometric_dimension()
             shape = (dim, dim)
 
@@ -316,14 +397,14 @@ class TensorElement(MixedElement):
             if symmetry == True:
                 symmetry = dict( ((i,j), (j,i)) for i in range(dim) for j in range(dim) if i > j )
             else:
-                ufl_assert(symmetry in (None, True),
+                ufl_assert(symmetry is None,
                           "Symmetry of tensor element cannot be specified unless shape has been specified.")
 
         # Compute all index combinations for given shape
         indices = compute_indices(shape)
 
         # Compute sub elements and mapping from indices to sub elements, accounting for symmetry
-        sub_element = FiniteElement(family, cell, degree)
+        sub_element = FiniteElement(family, cell, degree, quad_scheme)
         sub_elements = []
         sub_element_mapping = {}
         for index in indices:
@@ -344,7 +425,7 @@ class TensorElement(MixedElement):
         value_shape = shape + sub_element.value_shape()
 
         # Initialize element data
-        MixedElement.__init__(self, sub_elements, value_shape=value_shape)
+        super(TensorElement, self).__init__(sub_elements, value_shape=value_shape)
         self._family = family
         self._degree = degree
         self._sub_element = sub_element
@@ -353,8 +434,22 @@ class TensorElement(MixedElement):
         self._sub_element_mapping = sub_element_mapping
 
         # Cache repr string
-        self._repr = "TensorElement(%r, %r, %r, %r, %r)" % \
-            (self._family, self._cell, self._degree, self._shape, self._symmetry)
+        self._repr = "TensorElement(%r, %r, %r, %r, %r, %r)" % \
+            (self._family, self._cell, self._degree, self._shape, self._symmetry, quad_scheme)
+
+    def reconstruct(self, **kwargs):
+        kwargs["family"] = kwargs.get("family", self.family())
+        kwargs["cell"]   = kwargs.get("cell",   self.cell())
+        kwargs["degree"] = kwargs.get("degree", self.degree())
+
+        ufl_assert("shape" not in kwargs, "Cannot change shape in reconstruct.")
+
+        # Not sure about symmetry, but no use case I can see
+        ufl_assert("symmetry" not in kwargs, "Cannot change symmetry in reconstruct.")
+        kwargs["symmetry"] = self.symmetry()
+
+        kwargs["quad_scheme"] = kwargs.get("quad_scheme", self.quadrature_scheme())
+        return TensorElement(**kwargs)
 
     def extract_component(self, i):
         "Extract base component index and (simple) element for given component index"
@@ -405,15 +500,30 @@ class EnrichedElement(FiniteElementBase):
 
         degree = max(e.degree() for e in elements)
 
+        # We can allow the scheme not to be defined, but all defined should be equal
+        quad_schemes = [e.quadrature_scheme() for e in elements]
+        quad_schemes = [qs for qs in quad_schemes if qs is not None]
+        quad_scheme = quad_schemes[0] if quad_schemes else None
+        ufl_assert(all(qs == quad_scheme for qs in quad_schemes),\
+            "Quadrature scheme mismatch.")
+
         value_shape = elements[0].value_shape()
         ufl_assert(all(e.value_shape() == value_shape for e in elements), "Element value shape mismatch.")
 
         # Initialize element data
-        FiniteElementBase.__init__(self, "EnrichedElement", cell, degree, value_shape)
+        super(EnrichedElement, self).__init__("EnrichedElement", cell, degree, quad_scheme, value_shape)
 
         # Cache repr string
         self._repr = "EnrichedElement(%s)" % ", ".join(repr(e) for e in self._elements)
 
+    def reconstruct(self, **kwargs):
+        """Construct a new EnrichedElement object with some properties
+        replaced with new values."""
+        elements = [e.reconstruct(**kwargs) for e in self._elements]
+        if all(a == b for (a,b) in izip(elements, self._elements)):
+            return self
+        return EnrichedElement(*elements)
+
     def __str__(self):
         "Format as string for pretty printing."
         return "<%s>" % " + ".join(str(e) for e in self._elements)
@@ -427,24 +537,40 @@ class RestrictedElement(FiniteElementBase):
         ufl_assert(isinstance(element, FiniteElementBase), "Expecting a finite element instance.")
         from ufl.integral import Measure
         from ufl.geometry import Cell
-        ufl_assert(isinstance(domain, Measure) or isinstance(as_cell(domain), Cell),\
-            "Expecting a subdomain represented by a Measure or Cell instance.")
-        FiniteElementBase.__init__(self, "RestrictedElement", element.cell(), element.degree(), element.value_shape())
+        ufl_assert(isinstance(domain, Measure) or domain == "facet"\
+                   or isinstance(as_cell(domain), Cell),\
+            "Expecting a subdomain represented by a Measure, a Cell instance, or the string 'facet'.")
+        super(RestrictedElement, self).__init__("RestrictedElement", element.cell(),\
+            element.degree(), element.quadrature_scheme(), element.value_shape())
         self._element = element
 
-        # Just attach domain if it is a Measure
-        if isinstance(domain, Measure):
+        # Just attach domain if it is a Measure or Cell
+        if isinstance(domain, (Measure, Cell)):
             self._domain = domain
         else:
-            # Create Cell (if we get a string)
-            domain = as_cell(domain)
             # Check for facet and handle it
-            if domain.domain() == "facet":
-                domain = Cell(domain2facet[self.cell().domain()])
+            if domain == "facet":
+                cell = self.cell()
+                ufl_assert(not cell.is_undefined(), "Cannot determine facet cell of undefined cell.")
+                domain = Cell(domain2facet[cell.domain()])
+            else:
+                # Create Cell (if we get a string)
+                domain = as_cell(domain)
             self._domain = domain
 
-        self._repr = "RestrictedElement(%r, %r)" % (element, domain)
+        if isinstance(self._domain, Cell) and self._domain.is_undefined():
+            warning("Undefined cell as domain in RestrictedElement. Not sure if this is well defined.")
 
+        # Cache repr string
+        self._repr = "RestrictedElement(%r, %r)" % (self._element, self._domain)
+
+    def reconstruct(self, **kwargs):
+        """Construct a new EnrichedElement object with some properties
+        replaced with new values."""
+        element = self._element.reconstruct(**kwargs)
+        domain = kwargs.get("domain", self.domain())
+        return RestrictedElement(element=element, domain=domain)
+    
     def element(self):
         "Return the element which is restricted."
         return self._element
