@@ -1,9 +1,26 @@
 "The Form class."
 
-__authors__ = "Martin Sandve Alnes"
-__date__    = "2008-03-14 -- 2009-12-08"
-
-# Modified by Anders Logg, 2009.
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Anders Logg, 2009-2011.
+#
+# First added:  2008-03-14
+# Last changed: 2009-12-08
 
 from ufl.log import error
 from ufl.assertions import ufl_assert
@@ -15,7 +32,13 @@ from ufl.integral import Integral, Measure
 
 class Form(object):
     """Description of a weak form consisting of a sum of integrals over subdomains."""
-    __slots__ = ("_integrals", "_repr", "_hash", "_str", "_form_data")
+    __slots__ = ("_integrals",
+                 "_repr", "_hash", "_str", "_form_data", "_is_preprocessed",
+                 "cell_domains", "exterior_facet_domains", "interior_facet_domains")
+
+    # Note: cell_domains, exterior_facet_domains and interior_facet_domains
+    # are used by DOLFIN to pass data to the assembler. They can otherwise
+    # safely be ignored.
 
     def __init__(self, integrals):
         self._integrals = tuple(integrals)
@@ -24,13 +47,24 @@ class Form(object):
         self._repr = None
         self._hash = None
         self._form_data = None
+        self._is_preprocessed = False
+
+        # TODO: Can we attach non-ufl payloads in a more generic fashion?
+        # This seems prone to change with dolfin versions.
+        self.cell_domains = None
+        self.exterior_facet_domains = None
+        self.interior_facet_domains = None
 
     def cell(self):
+        c = None
         for itg in self._integrals:
-            c = itg.integrand().cell()
-            if c is not None:
-                return c
-        return None
+            d = itg.integrand().cell()
+            if d is not None:
+                c = d # Best we found so far
+                if not d.is_undefined():
+                    # Use the first fully defined cell we find
+                    break
+        return c
 
     def integral_groups(self):
         """Return a dict, which is a mapping from domains to integrals.
@@ -85,6 +119,23 @@ class Form(object):
         "Return form metadata (None if form has not been preprocessed)"
         return self._form_data
 
+    def compute_form_data(self,
+                          object_names=None,
+                          common_cell=None,
+                          element_mapping=None):
+        "Compute and return form metadata"
+        if self._form_data is None:
+            from ufl.algorithms.preprocess import preprocess
+            self._form_data = preprocess(self,
+                                         object_names=object_names,
+                                         common_cell=common_cell,
+                                         element_mapping=element_mapping)
+        return self.form_data()
+
+    def is_preprocessed(self):
+        "Check whether form is preprocessed"
+        return self._is_preprocessed
+
     def __add__(self, other):
         # --- Add integrands of integrals with the same measure
 
@@ -127,10 +178,10 @@ class Form(object):
         ufl_assert(is_python_scalar(scalar), "Only multiplication by scalar literals currently supported.")
         return Form([scalar*itg for itg in self._integrals])
 
-    def __mul__(self, function):
-        "The action of this form on the given function."
+    def __mul__(self, coefficient):
+        "The action of this form on the given coefficient."
         from ufl.formoperators import action
-        return action(self, function)
+        return action(self, coefficient)
 
     def __str__(self):
         if self._str is None:
