@@ -1,9 +1,25 @@
 "Various high level ways to transform a complete Form into a new Form."
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009
+#
+# First added:  2008-03-14
 # Last changed: 2010-09-07
 
 from ufl.log import error
@@ -71,23 +87,23 @@ def functional(form): # TODO: Does this make sense for anything other than testi
     form = expand_derivatives(form)
     return compute_form_functional(form)
 
-def action(form, function=None):
+def action(form, coefficient=None):
     """Given a bilinear form, return a linear form
-    with an additional function coefficient, representing
-    the action of the form on the function. This can be
+    with an additional coefficient, representing the
+    action of the form on the coefficient. This can be
     used for matrix-free methods."""
     form = as_form(form)
     form = expand_derivatives(form)
-    return compute_form_action(form, function)
+    return compute_form_action(form, coefficient)
 
-def energy_norm(form, function=None):
+def energy_norm(form, coefficient=None):
     """Given a bilinear form, return a linear form
-    with an additional function coefficient, representing
-    the action of the form on the function. This can be
+    with an additional coefficient, representing the
+    action of the form on the coefficient. This can be
     used for matrix-free methods."""
     form = as_form(form)
     form = expand_derivatives(form)
-    return compute_energy_norm(form, function)
+    return compute_energy_norm(form, coefficient)
 
 def adjoint(form):
     """Given a combined bilinear form, compute the adjoint
@@ -96,88 +112,88 @@ def adjoint(form):
     form = expand_derivatives(form)
     return compute_form_adjoint(form)
 
-def _handle_derivative_arguments(function, basis_function):
+def _handle_derivative_arguments(coefficient, argument):
     """Valid combinations:
     - Coefficient, Argument. Elements must match.
     - (Coefficient tuple,), Argument. Argument element must be a mixed element with subelements matching elements of the Coefficient tuple.
     """
 
-    if isinstance(function, Coefficient):
+    if isinstance(coefficient, Coefficient):
         # Place in generic tuple
-        functions = (function,)
+        coefficients = (coefficient,)
 
         # Get element
-        element = function.element()
+        element = coefficient.element()
 
-        # Create basis function if necessary
-        if basis_function is None:
-            basis_function = Argument(element)
-        ufl_assert(isinstance(basis_function, Argument),
-            "Expecting Argument instance, not %s." % type(basis_function))
-        ufl_assert(basis_function.element() == element,
-            "Basis function over wrong element supplied, "\
+        # Create Argument if necessary
+        if argument is None:
+            argument = Argument(element)
+        ufl_assert(isinstance(argument, Argument),
+            "Expecting Argument instance, not %s." % type(argument))
+        ufl_assert(argument.element() == element,
+            "Argument over wrong element supplied, "\
             "got %s but expecting %s." % \
-            (repr(basis_function.element()), repr(element)))
+            (repr(argument.element()), repr(element)))
 
         # Place in generic tuple
-        basis_functions = (basis_function,)
+        arguments = (argument,)
 
-    elif isinstance(function, (tuple, list)):
+    elif isinstance(coefficient, (tuple, list)):
         # Place in generic tuple
-        functions = function
+        coefficients = coefficient
 
-        # We got a tuple of functions, handle it as
-        # functions over components of a mixed element.
-        ufl_assert(all(isinstance(w, Coefficient) for w in functions),
+        # We got a tuple of coefficients, handle it as
+        # coefficients over components of a mixed element.
+        ufl_assert(all(isinstance(w, Coefficient) for w in coefficients),
             "Expecting a tuple of Coefficients to differentiate w.r.t.")
 
         # Create mixed element
-        elements = [w.element() for w in functions]
+        elements = [w.element() for w in coefficients]
         element = MixedElement(*elements)
 
-        # Create basis functions if necessary
-        if basis_function is None:
-            basis_function = Argument(element)
-        ufl_assert(isinstance(basis_function, Argument),
-            "Expecting Argument instance, not %s." % type(basis_function))
-        ufl_assert(basis_function.element() == element,
-            "Basis function over wrong element supplied, "\
+        # Create arguments if necessary
+        if argument is None:
+            argument = Argument(element)
+        ufl_assert(isinstance(argument, Argument),
+            "Expecting Argument instance, not %s." % type(argument))
+        ufl_assert(argument.element() == element,
+            "Arguments over wrong element supplied, "\
             "got %s but expecting %s." % \
-            (repr(basis_function.element()), repr(element)))
+            (repr(argument.element()), repr(element)))
 
         # Place in generic tuple
-        basis_functions = split(basis_function)
+        arguments = split(argument)
 
     else:
-        error("Expecting Coefficient instance or tuple of Coefficient instances, not %s." % type(function))
+        error("Expecting Coefficient instance or tuple of Coefficient instances, not %s." % type(coefficient))
 
     # Wrap and return generic tuples
-    functions       = Tuple(*functions)
-    basis_functions = Tuple(*basis_functions)
-    return functions, basis_functions
+    coefficients       = Tuple(*coefficients)
+    arguments = Tuple(*arguments)
+    return coefficients, arguments
 
-def derivative(form, function, basis_function=None):
+def derivative(form, coefficient, argument=None):
     """Given any form, compute the linearization of the
-    form with respect to the given discrete function.
-    The resulting form has one additional basis function
-    in the same finite element space as the function.
+    form with respect to the given Coefficient.
+    The resulting form has one additional Argument
+    in the same finite element space as the coefficient.
     A tuple of Coefficients may be provided in place of
     a single Coefficient, in which case the new Argument
     argument is based on a MixedElement created from this tuple."""
 
-    functions, basis_functions = _handle_derivative_arguments(function, basis_function)
+    coefficients, arguments = _handle_derivative_arguments(coefficient, argument)
 
     # Got a form? Apply derivatives to the integrands in turn.
     if isinstance(form, Form):
         integrals = []
         for itg in form._integrals:
-            fd = CoefficientDerivative(itg.integrand(), functions, basis_functions)
+            fd = CoefficientDerivative(itg.integrand(), coefficients, arguments)
             integrals.append(itg.reconstruct(fd))
         return Form(integrals)
 
     elif isinstance(form, Expr):
         # What we got was in fact an integrand
-        return CoefficientDerivative(form, functions, basis_functions)
+        return CoefficientDerivative(form, coefficients, arguments)
 
     error("Invalid argument type %s." % str(type(form)))
 
