@@ -1,32 +1,50 @@
 "Types for quantities computed from cell geometry."
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2009.
 # Modified by Kristian B. Oelgaard, 2009
-# Last changed: 2009-12-21
+#
+# First added:  2008-03-14
+# Last changed: 2011-05-08
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
-#from ufl.common import domain2dim, istr
 from ufl.common import istr
 from ufl.terminal import Terminal
 
 # --- Expression node types
 
 # Mapping from domain (cell) to dimension
-domain2dim = {None: None,
+domain2dim = {"cell1D": 1,
+              "cell2D": 2,
+              "cell3D": 3,
               "vertex": 0,
               "interval": 1,
               "triangle": 2,
               "tetrahedron": 3,
               "quadrilateral": 2,
-              "hexahedron": 3,
-              "facet": 0}
+              "hexahedron": 3}
 
 # Mapping from domain (cell) to facet
-domain2facet = {None: None,
+domain2facet = {"cell1D": "vertex",
+                "cell2D": "cell1D",
+                "cell3D": "cell2D",
                 "interval": "vertex",
                 "triangle": "interval",
                 "tetrahedron": "triangle",
@@ -34,8 +52,7 @@ domain2facet = {None: None,
                 "hexahedron": "quadrilateral"}
 
 # Number of facets associated with each domain
-domain2num_facets = {None: None,
-                     "interval": 2,
+domain2num_facets = {"interval": 2,
                      "triangle": 3,
                      "tetrahedron": 4,
                      "quadrilateral": 4,
@@ -61,7 +78,7 @@ class SpatialCoordinate(GeometricQuantity):
         self._repr = "SpatialCoordinate(%r)" % self._cell
 
     def shape(self):
-        d = self._cell.d
+        d = self._cell.geometric_dimension()
         if d == 1:
             return ()
         return (d,)
@@ -86,7 +103,7 @@ class FacetNormal(GeometricQuantity):
         self._repr = "FacetNormal(%r)" % self._cell
 
     def shape(self):
-        d = self._cell.d
+        d = self._cell.geometric_dimension()
         if d == 1:
             return ()
         return (d,)
@@ -162,57 +179,76 @@ class Space(object):
     __slots__ = ("_dimension",)
 
     def __init__(self, dimension):
-        #ufl_assert(isinstance(dimension, int), "Expecting integer.")
+        #ufl_assert(isinstance(dimension, int), "Expecting integer.") # FIXME: This is essential!
         self._dimension = dimension
 
     def dimension(self):
+        ufl_assert(isinstance(self._dimension, int), "No dimension defined!")
         return self._dimension
 
     def __str__(self):
         return "R%s" % istr(self._dimension)
 
     def __repr__(self):
-        return "Space(%s)" % istr(self._dimension)
+        return "Space(%r)" % (self._dimension,)
 
 class Cell(object):
     "Representation of a finite element cell."
-    __slots__ = ("_domain", "_degree", "_space", "_geometric_dimension",
-                 "_topological_dimension", "_repr", "d", "n", "x", "volume",
-                 "circumradius")
+    __slots__ = ("_domain", "_space", "_geometric_dimension",
+                 "_topological_dimension", "_repr", "_invalid",
+                 "d", "n", "x", "volume", "circumradius")
 
-    def __init__(self, domain, degree=1, space=None):
+    def __init__(self, domain, space=None):
         "Initialize basic cell description"
 
-        # Handle domain
-        ufl_assert(domain in domain2dim, "Invalid domain %s." % (domain,))
-        self._domain = domain
-        self._topological_dimension = domain2dim[self._domain]
-
-        # Handle degree
-        ufl_assert(isinstance(degree, int) and degree >= 1, "Invalid degree '%r'." % (degree,))
-        if degree != 1: # TODO: Remove warning when implemented
-            warning("Note: High order geometries are not implemented in the form compilers yet.")
-        self._degree = degree
-
-        # Get geometric dimension
-        if space is None:
-            space = Space(self._topological_dimension)
-        ufl_assert(isinstance(space, Space), "Expecting a Space instance, not '%r'" % (space,))
-        self._space = space
-        self._geometric_dimension = self._space.dimension()
-
-        ufl_assert(self._topological_dimension <= self._geometric_dimension,
-            "Cannot embed a %sD cell in %s" % (istr(self._topological_dimension), self._space))
-
-        # TODO: Make self.d a property, deprecate or make valid
-        #       only in this case. Don't use inside UFL!
-        if self._topological_dimension == self._geometric_dimension:
-            self.d = self._geometric_dimension
+        # Check for valid domain. We allow None to support PyDOLFIN
+        # integration features. This is a bit dangerous because
+        # several things in UFL become undefined, but it is a very
+        # good and important feature to have so don't even think about
+        # removing it! ;-)
+        if domain is None:
+            self._invalid = True
         else:
-            self.d = None
+            ufl_assert(domain in domain2dim, "Invalid domain %s." % (domain,))
+            self._invalid = False
+        self._domain = domain
+
+        # Don't compute quantities that are undefined
+        if self._invalid:
+            # Used in repr string below
+            self._space = None
+        else:
+            # The topological dimension is defined by the cell type
+            dim = domain2dim[self._domain]
+            self._topological_dimension = dim
+
+            # The space dimension defaults to equal the topological dimension if undefined
+            space = Space(dim) if space is None else space
+            ufl_assert(isinstance(space, Space),
+                       "Expecting a Space instance, not '%r'" % (space,))
+            self._space = space
+
+            # The geometric dimension is defined by the space
+            self._geometric_dimension = space.dimension()
+
+            # Check for consistency in dimensions.
+            # NB! Note that the distinction between topological
+            # and geometric dimensions has yet to be used in
+            # practice, so don't trust it too much :)
+            ufl_assert(self._topological_dimension <= self._geometric_dimension,
+                       "Cannot embed a %sD cell in %s" %\
+                           (istr(self._topological_dimension), self._space))
+
+            # Attach a cell dimension for use in code.
+            # TODO: Make self.d a property, deprecate or make valid
+            #       only in this case. Don't use inside UFL!
+            if self._topological_dimension == self._geometric_dimension:
+                self.d = self._geometric_dimension
+            else:
+                self.d = None # TODO: Make this a property so user code can fail instead of silently getting None
 
         # Cache repr string
-        self._repr = "Cell(%r, %r, %r)" % (self._domain, self._degree, self._space)
+        self._repr = "Cell(%r, %r)" % (self._domain, self._space)
 
         # Attach expression nodes derived from this cell
         self.n = FacetNormal(self)
@@ -224,34 +260,39 @@ class Cell(object):
         #self.hmax = MeshSizeMax(self)
 
     def geometric_dimension(self):
+        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
         return self.d
 
     def topological_dimension(self):
+        ufl_assert(not self._invalid, "An invalid cell has no dimensions.")
         return self.d
 
+    def is_undefined(self):
+        return self._invalid
+
     def domain(self):
+        ufl_assert(not self._invalid, "An invalid cell has no domain.")
         return self._domain
 
-    def degree(self):
-        return self._degree
-
     def num_facets(self):
+        ufl_assert(not self._invalid, "An invalid cell has no facets.")
         return domain2num_facets[self._domain]
 
     def facet_domain(self):
+        ufl_assert(not self._invalid, "An invalid cell has no facet domains.")
         return domain2facet[self._domain]
 
     def __eq__(self, other):
-        return isinstance(other, Cell) and self._domain == other._domain and self._degree == other._degree
+        return isinstance(other, Cell) and repr(self) == repr(other)
 
     def __ne__(self, other):
         return not self == other
 
     def __hash__(self):
-        return hash(("Cell", self._domain, self._degree))
+        return hash(repr(self))
 
     def __str__(self):
-        return "<%s of degree %d>" % (istr(self._domain), self._degree)
+        return "<%s cell in %s>" % (istr(self._domain), istr(self._space))
 
     def __repr__(self):
         return self._repr
