@@ -1,9 +1,26 @@
 """The Integral class."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-06-19"
-
-# Modified by Anders Logg, 2008-2009.
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by Anders Logg, 2008-2009
+#
+# First added:  2008-03-14
+# Last changed: 2009-06-19
 
 from ufl.log import error
 from ufl.assertions import ufl_assert
