@@ -1,12 +1,27 @@
 """This module provides functions used by the UFL implementation to
 output messages. These may be redirected by the user of UFL."""
 
-__author__ = "Martin Sandve Alnaes and Anders Logg"
-__date__ = "2005-02-04 -- 2009-08-17"
-__copyright__ = "Copyright (C) 2005-2009 Anders Logg and Martin Sandve Alnaes"
-__license__  = "GNU GPL version 3 or any later version"
-
+# Copyright (C) 2005-2011 Anders Logg and Martin Sandve Alnaes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Johan Hake, 2009.
+#
+# First added:  2005-02-04
+# Last changed: 2011-04-12
 
 import sys
 import types
@@ -22,11 +37,6 @@ __all__ = log_functions + ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL", "Log
 # Import default log levels
 from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL
 
-# Base class for UFL exceptions
-class UFLException(Exception):
-    "Base class for UFL exceptions"
-    pass
-
 # This is used to override emit() in StreamHandler for printing without newline
 def emit(self, record):
     message = self.format(record)
@@ -42,9 +52,10 @@ GREEN = "\033[1;37;32m%s\033[0m"
 # Logger class
 class Logger:
 
-    def __init__(self, name):
+    def __init__(self, name, exception_type=Exception):
         "Create logger instance."
         self._name = name
+        self._exception_type = exception_type
 
         # Set up handler
         h = logging.StreamHandler(sys.stdout)
@@ -70,7 +81,7 @@ class Logger:
         # Set prefix
         self._prefix = ""
 
-    def add_logfile(self, filename=None, mode="a"):
+    def add_logfile(self, filename=None, mode="a", level=DEBUG):
         if filename is None:
             filename = "%s.log" % self._name
         if filename in self._logfiles:
@@ -78,7 +89,7 @@ class Logger:
             return
         h = logging.FileHandler(filename, mode)
         h.emit = types.MethodType(emit, h, h.__class__)
-        h.setLevel(DEBUG)
+        h.setLevel(level)
         self._log.addHandler(h)
         self._logfiles[filename] = h
         return h
@@ -121,7 +132,7 @@ class Logger:
     def error(self, *message):
         "Write error message and raise an exception."
         self._log.error(*message)
-        raise UFLException(self._format_raw(*message))
+        raise self._exception_type(self._format_raw(*message))
 
     def begin(self, *message):
         "Begin task: write message and increase indentation level."
@@ -194,7 +205,12 @@ class Logger:
 
 #--- Set up global log functions ---
 
-ufl_logger = Logger("UFL")
+# Base class for UFL exceptions
+class UFLException(Exception):
+    "Base class for UFL exceptions"
+    pass
+
+ufl_logger = Logger("UFL", UFLException)
 
 for foo in log_functions:
     exec("%s = ufl_logger.%s" % (foo, foo))
