@@ -1,15 +1,51 @@
 """This module provides basic mathematical functions."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-04-21"
-
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
 # Modified by Anders Logg, 2008
+#
+# First added:  2008-03-14
+# Last changed: 2009-04-21
 
 import math
 from ufl.assertions import ufl_assert
 from ufl.expr import Operator
 from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue
 
+"""
+TODO: Include additional functions available in <cmath> (need derivatives as well):
+
+Trigonometric functions:
+atan2	 Compute arc tangent with two parameters (function)
+
+Hyperbolic functions:
+cosh	 Compute hyperbolic cosine (function)
+sinh	 Compute hyperbolic sine (function)
+tanh	 Compute hyperbolic tangent (function)
+
+Exponential and logarithmic functions:
+log10	 Compute common logarithm (function)
+
+TODO: Include bessel functions, need non-standard library implementation in generated code.
+
+TODO: Other special functions?
+"""
+
 #--- Function representations ---
 
 class MathFunction(Operator):
