@@ -1,7 +1,24 @@
 """Compound tensor algebra operations."""
 
-__authors__ = "Martin Sandve Alnes"
-__date__ = "2008-03-14 -- 2009-04-02"
+# Copyright (C) 2008-2011 Martin Sandve Alnes
+#
+# This file is part of UFL.
+#
+# UFL is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# UFL is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with UFL.  If not, see <http://www.gnu.org/licenses/>.
+#
+# First added:  2008-03-14
+# Last changed: 2009-04-02
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
@@ -126,9 +143,12 @@ class Outer(CompoundTensorOperator):
     __slots__ = ("_a", "_b", "_free_indices", "_index_dimensions")
 
     def __new__(cls, a, b):
+        ash, bsh = a.shape(), b.shape()
         if isinstance(a, Zero) or isinstance(b, Zero):
             free_indices, index_dimensions = merge_indices(a, b)
-            return Zero(a.shape() + b.shape(), free_indices, index_dimensions)
+            return Zero(ash + bsh, free_indices, index_dimensions)
+        if ash == () or bsh == ():
+            return a * b
         return CompoundTensorOperator.__new__(cls)
 
     def __init__(self, a, b):
@@ -159,10 +179,13 @@ class Inner(CompoundTensorOperator):
     __slots__ = ("_a", "_b", "_free_indices", "_index_dimensions")
 
     def __new__(cls, a, b):
-        ufl_assert(a.shape() == b.shape(), "Shape mismatch.")
+        ash, bsh = a.shape(), b.shape()
+        ufl_assert(ash == bsh, "Shape mismatch.")
         if isinstance(a, Zero) or isinstance(b, Zero):
             free_indices, index_dimensions = merge_indices(a, b)
             return Zero((), free_indices, index_dimensions)
+        if ash == ():
+            return a*b
         return CompoundTensorOperator.__new__(cls)
 
     def __init__(self, a, b):
@@ -195,15 +218,17 @@ class Dot(CompoundTensorOperator):
     __slots__ = ("_a", "_b", "_free_indices", "_index_dimensions")
 
     def __new__(cls, a, b):
-        ufl_assert(a.rank() >= 1 and b.rank() >= 1,
-            "Dot product requires non-scalar arguments, got arguments with ranks %d and %d." % \
-            (a.rank(), b.rank()))
-        
+        ar, br = a.rank(), b.rank()
+        ufl_assert((ar >= 1 and br >= 1) or (ar == 0 and br == 0),
+            "Dot product requires non-scalar arguments, "\
+            "got arguments with ranks %d and %d." % \
+            (ar, br))
         if isinstance(a, Zero) or isinstance(b, Zero):
             shape = a.shape()[:-1] + b.shape()[1:]
             free_indices, index_dimensions = merge_indices(a, b)
             return Zero(shape, free_indices, index_dimensions)
-        
+        if ar == 0 and br == 0:
+            return a * b
         return CompoundTensorOperator.__new__(cls)
 
     def __init__(self, a, b):
@@ -234,23 +259,21 @@ class Cross(CompoundTensorOperator):
     __slots__ = ("_a", "_b", "_free_indices", "_index_dimensions")
 
     def __new__(cls, a, b):
+        ash, bsh = a.shape(), b.shape()
+        ufl_assert(len(ash) == 1 and ash == bsh,
+            "Cross product requires arguments of rank 1.")
+
         if isinstance(a, Zero) or isinstance(b, Zero):
-            warning("Returning zero from Cross not implemented.")
+            free_indices, index_dimensions = merge_indices(a, b)
+            return Zero(ash, free_indices, index_dimensions)
+
         return CompoundTensorOperator.__new__(cls)
 
     def __init__(self, a, b):
         CompoundTensorOperator.__init__(self)
-        ufl_assert(a.rank() == 1 and b.rank() == 1,
-            "Cross product requires arguments of rank 1.")
         self._a = a
         self._b = b
-        ai = self._a.free_indices()
-        bi = self._b.free_indices()
-        ufl_assert(not (set(ai) ^ set(bi)),
-            "Not expecting repeated indices in outer product.") 
-        self._free_indices = ai+bi
-        self._index_dimensions = dict(a.index_dimensions())
-        self._index_dimensions.update(b.index_dimensions())
+        self._free_indices, self._index_dimensions = merge_indices(a, b)
     
     def operands(self):
         return (self._a, self._b)
@@ -315,6 +338,8 @@ class Determinant(CompoundTensorOperator):
             "Not expecting free indices in determinant.")
         if isinstance(A, Zero):
             return Zero((), A.free_indices(), A.index_dimensions())
+        if r == 0:
+            return A
         return CompoundTensorOperator.__new__(cls)
 
     def __init__(self, A):
@@ -343,14 +368,25 @@ class Determinant(CompoundTensorOperator):
 class Inverse(CompoundTensorOperator):
     __slots__ = ("_A",)
 
-    def __init__(self, A):
-        CompoundTensorOperator.__init__(self)
+    def __new__(cls, A):
         sh = A.shape()
         r = len(sh)
-        ufl_assert(r == 0 or r == 2, "Inverse of tensor with rank != 2 or 0 is undefined.")
-        ufl_assert(r == 0 or sh[0] == sh[1],
-            "Cannot take inverse of rectangular matrix with dimensions %s." % repr(sh))
-        ufl_assert(not A.free_indices(), "Not expecting free indices in Inverse.")
+        if A.free_indices():
+            error("Not expecting free indices in Inverse.")
+        if isinstance(A, Zero):
+            error("Division by zero!")
+
+        if r == 0:
+            return 1 / A
+
+        if r != 2:
+            error("Inverse of tensor with rank != 2 is undefined.")
+        if sh[0] != sh[1]:
+            error("Cannot take inverse of rectangular matrix with dimensions %s." % repr(sh))
+        return CompoundTensorOperator.__new__(cls)
+
+    def __init__(self, A):
+        CompoundTensorOperator.__init__(self)
         self._A = A
     
     def operands(self):
@@ -380,8 +416,9 @@ class Cofactor(CompoundTensorOperator):
         ufl_assert(len(sh) == 2, "Cofactor of tensor with rank != 2 is undefined.")
         ufl_assert(sh[0] == sh[1], "Cannot take cofactor of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Cofactor.")
+        ufl_assert(not isinstance(A, Zero), "Cannot take cofactor of zero matrix.")
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
     
@@ -403,16 +440,20 @@ class Cofactor(CompoundTensorOperator):
 class Deviatoric(CompoundTensorOperator):
     __slots__ = ("_A",)
 
-    def __init__(self, A):
-        CompoundTensorOperator.__init__(self)
+    def __new__(cls, A):
         sh = A.shape()
-        r = len(sh)
-        ufl_assert(r == 2, "Deviatoric part of tensor with rank != 2 is undefined.")
+        ufl_assert(len(sh) == 2, "Deviatoric part of tensor with rank != 2 is undefined.")
         ufl_assert(sh[0] == sh[1],
             "Cannot take deviatoric part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Deviatoric.")
+        if isinstance(A, Zero):
+            return Zero(A.shape(), A.free_indices(), A.index_dimensions())
+        return CompoundTensorOperator.__new__(cls)
+
+    def __init__(self, A):
+        CompoundTensorOperator.__init__(self)
         self._A = A
-    
+
     def operands(self):
         return (self._A, )
     
@@ -434,14 +475,18 @@ class Deviatoric(CompoundTensorOperator):
 class Skew(CompoundTensorOperator):
     __slots__ = ("_A",)
 
-    def __init__(self, A):
-        CompoundTensorOperator.__init__(self)
+    def __new__(cls, A):
         sh = A.shape()
-        r = len(sh)
-        ufl_assert(r == 2, "Skew symmetric part of tensor with rank != 2 is undefined.")
+        ufl_assert(len(sh) == 2, "Skew symmetric part of tensor with rank != 2 is undefined.")
         ufl_assert(sh[0] == sh[1],
             "Cannot take skew part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Skew.")
+        if isinstance(A, Zero):
+            return Zero(A.shape(), A.free_indices(), A.index_dimensions())
+        return CompoundTensorOperator.__new__(cls)
+
+    def __init__(self, A):
+        CompoundTensorOperator.__init__(self)
         self._A = A
     
     def operands(self):
@@ -465,14 +510,18 @@ class Skew(CompoundTensorOperator):
 class Sym(CompoundTensorOperator):
     __slots__ = ("_A",)
 
-    def __init__(self, A):
-        CompoundTensorOperator.__init__(self)
+    def __new__(cls, A):
         sh = A.shape()
-        r = len(sh)
-        ufl_assert(r == 2, "Symmetric part of tensor with rank != 2 is undefined.")
+        ufl_assert(len(sh) == 2, "Symmetric part of tensor with rank != 2 is undefined.")
         ufl_assert(sh[0] == sh[1],
             "Cannot take symmetric part of rectangular matrix with dimensions %s." % repr(sh))
         ufl_assert(not A.free_indices(), "Not expecting free indices in Sym.")
+        if isinstance(A, Zero):
+            return Zero(A.shape(), A.free_indices(), A.index_dimensions())
+        return CompoundTensorOperator.__new__(cls)
+
+    def __init__(self, A):
+        CompoundTensorOperator.__init__(self)
         self._A = A
     
     def operands(self):
